<?php
/**
 * This is the Base class that every Controller must extend
 *
 * @author Macdonald Terrence Robinson
 * @package Core
 *
 */
class MainController
{
  public $template = '';
  public $templateVars = array();
  
  /** @var CMS */
  public $cms = NULL;

  public function __construct()
  {
    if( Config::$enableEditInPlace )
      $this->cms = $this->loadPlugin('CMS');
    
    $this->setDefaultFrameworkCode();

    Session::set('siteURL', Config::$siteURL);
    Session::set('siteURL', Config::$siteURL);
    Session::set('siteDir', Config::$siteDir);
    Session::set('writableDir', Config::$writableDir);
    Session::set('editInPlaceDir', Config::$editInPlaceDir);

    $pageTemplate = Config::$defaultTemplate;

    if(SiteMap::getCurrentNode() != NULL ) {
      $tmpPageTemplate = (string)SiteMap::getAttr( SiteMap::getCurrentNode(), 'pageTemplate' );

      if(($tmpPageTemplate !== '') && (strtolower($tmpPageTemplate) !== 'default' ))
        $pageTemplate = $tmpPageTemplate;
    }

    $this->setTemplate($pageTemplate);

    $this->addJSFile('js/jquery.js');
    $this->addJSFile('js/fckeditor/fckeditor.js');
    $this->addJSFile('js/jquery-ui.js');
    $this->addCSSFile('js/jquery-ui-themes/ui-lightness/jquery-ui-1.7.2.custom.css');

    $this->addJSFile('js/framework.js');
  }


  /**
   * Handles the Routing if the node cannot be found in the SiteMap
   * This handles the transfer of control to the CMS plugin
   * @return void
   */
  public function catchAll()
  {
    if(Router::$function == Config::$adminFunction)
      return $this->cms->start();
    else if( $this->getCurrentNode() == NULL )
    {
      if($this->isAjaxRequest())
        $this->returnContent('Page Not Found');
      else
        $this->redirectToHome();            
    }
    if( SiteMap::canDisplayNode( SiteMap::getCurrentNode() ) )
      $this->returnContent( $this->loadMainContent(Router::$function) , '' );
    else
      $this->redirectToHome();

  }

  /**
   * Generates an HTML List for the Language Switcher Nav based on the parameters given
   * @return string
   */
  public function generateLocalizationNav($ULId, $spacer = '<span class="spacer">|</span>', $addToEndOnEachLI='')
  {
    $newlist = Config::$siteLanguages;

    $controller = Router::$controller;
    $function = Router::$function;

    foreach ($newlist as $key=>$value) {
      $attributes = array();
      $attributes['href']= URI::getURL(true).'?lang='.$value;

      if(Config::$siteLanguage == $value)
        $attributes['class'] = 'selected';

      Utility::prepareArrayItem($newlist, $key, $value, $attributes);
    }

    $html = HTML::generateList($newlist, 'ul', $ULId, $spacer, $addToEndOnEachLI);

    return $html;
  }

  /**
   * Checks to see if the current request is an Ajax request or not
   * NOTE: This will only work if all ajax requests contain the query string 'ajaxRequest'
   * @return bool
   */
  public function isAjaxRequest()
  {
    if( Get::isExists('ajaxRequest') || Post::isExists('ajaxRequest') )
      return true;

    return false;
  }

  /**
   * Redirect to the Home Page
   * @return void
   */
  public function redirectToHome()
  {
    $this->redirectTo( Config::$defaultController, Config::$defaultFunction );
  }

  /**
   * Redirect to a node which matches the controller-function pair
   * @return void
   */
  public function redirectTo($controller, $function, $queryString='')
  {
    if( ($queryString != '') &&(!stristr($queryString, '?')) )
      $queryString = '?'.$queryString;

    $node = SiteMap::getNode( $controller, $function );

    if($node != NULL)
    {
      if( !$this->isAjaxRequest() )
        header('Location:'.SiteMap::getURL($node).$queryString );
      else
        $this->returnContent('<script>window.location.href="'.SiteMap::getURL($node).$queryString.'"</script>');
    }
  }

  /**
   * Set the template for the current page
   * @return void
   */
  public function setTemplate($templateName)
  {
    $this->templateVars['templatePath'] = dirname($this->getTemplatePath($templateName)).'/';
    $this->templateVars['templateURL'] = Config::$siteURL.$this->templateVars['templatePath'];
    $this->template = $templateName;
  }

  /**
   * Set Custom Template Variables which will be used in the parser
   * example: array( 'myName'=>'Mac' ); will replace all instances of {myName} with 'Mac'
   * @return void
   */
  public function setCustomVariables(array $templateVariables)
  {
    $this->customVariables = $templateVariables;
  }

  /**
   * Load a template
   * @return string
   */
  public function loadTemplate($templateName, array $keyValuePair = array())
  {
    $filePath = $this->getTemplatePath($templateName);
    
    $keyValuePair = array_merge($this->templateVars, $keyValuePair);

    $data = Parser::compileAndParse($filePath, $keyValuePair);

    if( Config::$enableCaching )
      $cacheData = Cache::saveCache($data);

    return $data;
  }

  /**
   * Display a Template
   * @return string
   */
  public function displayTemplate($templateName, array $keyValuePair = array())
  {
    if($templateName == '')
    {
      ErrorHandler::runtimeError('No template set, use function $this->setTemplate( $templateName )');
      return false;
    }

    echo $this->loadTemplate($templateName, $keyValuePair);
  }

  /**
   * Load a Model
   * @return BaseModel
   */
  public function loadModel($modelName)
  {
    $filePath = $this->getModelPath($modelName);
    return Master::loadAndInitClass($modelName, array(),'',Config::$modelsDir);
  }

  /**
   * Gets the path to the template
   * @return string
   */
  public function getTemplatePath($templateName)
  {
    $path = Config::$templatesDir.$templateName.'/index.php';

    $errorMessage = 'Error Loading Template ( '.$path.' )';
    FileSystem::fileMustExist($path, $errorMessage);

    return $path;
  }

  /**
   * Gets the path to the model
   * @return string
   */
  public function getModelPath($modelName)
  {
    $path = Config::$modelsDir.ucfirst($modelName).'.php';

    $errorMessage = 'Error Loading Model ( '.$path.' )';
    FileSystem::fileMustExist($path, $errorMessage);

    return $path;
  }

  /**
   * Gets the path to the plugin
   * @return string
   */
  public function getPluginPath($pluginName)
  {
    $path = Config::$pluginsDir.Config::$directorySeperator.$pluginName.Config::$directorySeperator.'controllers'.Config::$directorySeperator.ucfirst($pluginName).'.php';

    $errorMessage = 'Error Loading Plugin ( '.$path.' )';
    FileSystem::fileMustExist($path, $errorMessage);

    return $path;
  }

  /**
   * Gets the path to the Main Content
   * @return string
   */
  public function getMainContentPath($mainContentName)
  {
    $mainContentsDir = Config::$mainContentsDir;
    return $this->getContentFilePath($mainContentsDir, $mainContentName);
  }

  /**
   * Gets the path to the Content Bucket
   * @return string
   */
  public function getContentBucketPath($contentBucketName, $controller = '', $function = '', $editable=true, $uniquePerPage=true)
  {
    $contentBucketsDir = Config::$contentBucketsDir;

    if($uniquePerPage)
      $contentBucketName = $function.'-'.$contentBucketName;

    return $this->getContentFilePath($contentBucketsDir, $contentBucketName);
  }

  private function getContentFilePath($dir, $contentName)
  {
    $languageDir = Config::getLanguagePath(Config::$siteLanguage);
    $path = $languageDir.$dir.$contentName.'.php';

    $editInPlacePath = $this->getEditInPlacePath($dir, $contentName);

    if( FileSystem::fileExists( $editInPlacePath ) )
      $path = $editInPlacePath;
    else if( !FileSystem::fileExists( $path ) )
      $path = Config::getLanguagePath('common').$dir.$contentName.'.php';

    return $path;
  }


  /**
   * Gets the path to the Language Directory
   * @return string
   */
  public function getLanguageDir()
  {
    return Config::$editInPlaceDir.Config::$siteLanguage.'/';
  }

  /**
   * Gets the Edit In Place path to the Content
   * @return string
   */
  public function getEditInPlacePath($dir, $contentName)
  {
    if( !stristr($dir, '/') )
      $dir .= '/';

    $path = $this->getLanguageDir().$dir.$contentName.'.editInPlace.php';
    return $path;
  }

  /**
   * Gets the Main Content of a node
   * @return string
   */
  public function loadMainContent($mainContentName, array $customVariables = array(), $editable = true)
  {
    $filePath = $this->getMainContentPath($mainContentName);
    $parseSnippets = true;

    if( $this->isAjaxRequest() )
      $parseSnippets = false;

    $content = Parser::compileAndParse($filePath, $customVariables, $parseSnippets);    
    
    $comments = '';

    if ( isset($this->cms)  && ( !$this->isAjaxRequest() ))
    {      
      if((SiteMap::getCurrentNode() !=null) && ((string)SiteMap::getAttr(SiteMap::getCurrentNode(), 'allowVisitorComments')) == 'on' )
        $comments .= $this->cms->_getCommentsForm(Router::$controller, Router::$function);
      
      if((SiteMap::getCurrentNode() !=null) && ((string)SiteMap::getAttr(SiteMap::getCurrentNode(), 'showVisitorComments')) == 'on' )
        $comments .= $this->cms->_getComments(Router::$controller, Router::$function);
    }
    if(($editable) && ( isset($this->cms ) && ( $this->cms->isAdminLoggedIn() ) ))
    {
      $content = '
            <div class="editable" contentType="mainContents" contentName="'.$mainContentName.'">
              '.$content.$comments.'
            </div>
        ';
    }
    else
    {
      $content = $content.$comments;
    }
    return $content;
  }

  /**
   * Gets the Content Bucket based on the parameter values
   * @return string
   */
  public function loadContentBucket($contentBucketName, array $customVariables = array(), $editable = true, $uniquePerPage = true, $controller='', $function='')
  {    
    $filePath = $this->getContentBucketPath($contentBucketName, $controller, $function, $editable, $uniquePerPage);

    $parseSnippets = true;

    if( $this->isAjaxRequest() )
      $parseSnippets = false;

    $content = Parser::compileAndParse($filePath, $customVariables, $parseSnippets);
    
    $uniquePerPageText = 'true';

    if($uniquePerPage == false)
      $uniquePerPageText = 'false';
    
    if(($editable) && ( isset($this->cms ) && ( $this->cms->isAdminLoggedIn() ) ))
    {
      $content = '
        <div class="editable" contentType="contentBuckets" contentName="'.$contentBucketName.'" uniquePerPage="'.$uniquePerPageText.'">
          '.$content.'
        </div>
        ';
    }
    
    return $content;
  }

  /**
   * Loads and initializes a Plugin and returns the instance
   * @return BasePlugin
   */
  public function loadPlugin($pluginName)
  {
    $filePath = $this->getPluginPath($pluginName);
    return Master::loadAndInitClass($pluginName, array($this), $filePath);
  }

  /**
   * Gets the current node
   * @return BasePlugin
   */
  public function getCurrentNode()
  {
    return SiteMap::getCurrentNode();
  }

  /**
   * Displays the current node and applies the template
   * @return void
   */
  public function display($templateVariables = array())
  {
    $this->displayTemplate($this->template, $templateVariables);    
  }

  /**
   * Sets the protocol for the current page
   * @return void
   */
  public function setProtocol( $protocol )
  {
    Router::forceToProtocol($protocol);
  }

  /**
   * Allows you to add Raw code to the {frameworkCode} template variable
   * @return void
   */
  public function addToFrameworkCode($code, $topMost = false)
  {
    if($topMost)
      $this->templateVars['frameworkCode'] = $code.$this->templateVars['frameworkCode'];
    else
      $this->templateVars['frameworkCode'] .= $code;
  }

  /**
   * Sets the default {frameworkCode} value
   * @return void
   */
  public function setDefaultFrameworkCode()
  {
    $siteURL = Config::$siteURL;

    if( Config::$appendIndexPhpToURL )
    {
      if( !stristr($siteURL, 'index.php/') )
        $siteURL = $siteURL.'index.php/';
    }

    $isAdminLoggedIn = isset($this->cms) ? $this->cms->isAdminLoggedIn() : false;
    $isAdminLoggedInStr = ($isAdminLoggedIn) ? 'true' : 'false';

    $this->templateVars['frameworkCode'] = '
      <script type="text/javascript">
        var framework = {siteURL: "'.$siteURL.'",adminFunction: "'.Config::$adminFunction.'", isAdminLoggedIn: '.$isAdminLoggedInStr.', enableEditInPlace: '.((Config::$enableEditInPlace)? 'true': 'false').', currentController : "'.Router::$controller.'", currentFunction: "'.Router::$function.'", currentURL: "'.URI::getURL(true).'", queryString: "'.URI::getQueryString().'" }
      </script>';
  }

  /**
   * Allows you to add Raw JS code to the {frameworkCode} template variable
   * @return void
   */
  public function addRawJS( $jsCode )
  {
    $code = '
      <script type="text/javascript">
        '.$jsCode.'
      </script>';
    $this->addToFrameworkCode($code);
  }

  /**
   * Allows you to be able to add JS files from the Backend
   * @return bool
   */
  public function addJSFile($jsFilePath, $topMost = false)
  {
    $isLocal = (stristr($jsFilePath, '://') == '') ? true: false;

    $fileExists = false;

    if($isLocal) {
      if(is_file(Config::$siteDir.$jsFilePath)) {
        $jsFilePath = Config::$siteURL.$jsFilePath;
        $fileExists = true;
      }
    }
    else
      $fileExists = true;

    if( $fileExists ) {
      $code = '
        <script type="text/javascript" src="'.$jsFilePath.'"></script>';
      $this->addToFrameworkCode($code, $topMost);
    }

    return $fileExists;
  }

  /**
   * Allows you to be able to add JS files from the Backend
   * @return bool
   */
  public function addCSSFile($cssFilePath, $topMost = false)
  {
    $isLocal = (stristr($cssFilePath, '://') == '') ? true: false;
    $fileExists = false;

    if($isLocal)
    {
      if(is_file(Config::$siteDir.$cssFilePath))
      {
        $cssFilePath = Config::$siteURL.$cssFilePath;
        $fileExists = true;
      }
    }
    else
      $fileExists = true;

    if( $fileExists ) {
      $code = '
        <link type="text/css" rel="stylesheet" href="'.$cssFilePath.'" ></link>';

      $this->addToFrameworkCode($code);
    }

    return $fileExists;
  }

  /**
   * Check to see if the current page was a post or not
   * @return bool
   */
  public function IsPost()
  {
    if( count($_POST)>0 )
      return true;

    return false;
  }

  /**
   * Check to see if the current request had any files
   * @return bool
   */
  public function HasFiles()
  {
    if( isset($_FILES) && count($_FILES)>0 )
      return true;

    return false;
  }

  /**
   * Returns content based on the parameter values
   * If it is an ajax request it returns the content
   * else it applies the template and displays it
   * @return void | string
   */
  public function returnContent($content, $ajaxContent='', $returnRaw = false)
  {
    if($this->isAjaxRequest())
    {
      @header('Content-Type: text/html; charset='.Config::$charset);

      if($ajaxContent == '')
        echo $content;
      else
        echo $ajaxContent;

      exit;
    }
    else {
      if($returnRaw)
        return $content;
      else
      {
        @header('Content-Type: text/html; charset='.Config::$charset);

        $this->templateVars['mainContent'] = $content;
        $this->display($this->templateVars);
      }
    }
  }

}
?>
